
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>
#include <clapper/clapper-visibility.h>

G_BEGIN_DECLS

/* enumerations from "clapper-enums.h" */

CLAPPER_API
GType clapper_player_state_get_type (void);
#define CLAPPER_TYPE_PLAYER_STATE (clapper_player_state_get_type())

CLAPPER_API
GType clapper_player_seek_method_get_type (void);
#define CLAPPER_TYPE_PLAYER_SEEK_METHOD (clapper_player_seek_method_get_type())

CLAPPER_API
GType clapper_player_message_destination_get_type (void);
#define CLAPPER_TYPE_PLAYER_MESSAGE_DESTINATION (clapper_player_message_destination_get_type())

CLAPPER_API
GType clapper_queue_progression_mode_get_type (void);
#define CLAPPER_TYPE_QUEUE_PROGRESSION_MODE (clapper_queue_progression_mode_get_type())

CLAPPER_API
GType clapper_marker_type_get_type (void);
#define CLAPPER_TYPE_MARKER_TYPE (clapper_marker_type_get_type())

CLAPPER_API
GType clapper_stream_type_get_type (void);
#define CLAPPER_TYPE_STREAM_TYPE (clapper_stream_type_get_type())

CLAPPER_API
GType clapper_discoverer_discovery_mode_get_type (void);
#define CLAPPER_TYPE_DISCOVERER_DISCOVERY_MODE (clapper_discoverer_discovery_mode_get_type())

CLAPPER_API
GType clapper_enhancer_param_flags_get_type (void);
#define CLAPPER_TYPE_ENHANCER_PARAM_FLAGS (clapper_enhancer_param_flags_get_type())

CLAPPER_API
GType clapper_reactable_item_updated_flags_get_type (void);
#define CLAPPER_TYPE_REACTABLE_ITEM_UPDATED_FLAGS (clapper_reactable_item_updated_flags_get_type())

G_END_DECLS

/* Generated data ends here */

